     //////////////////////////////////////////////////////////////////////
     // The contents of this file are subject to the Mozilla Public License
     // Version 1.1 (the "License"); you may not use this file except in
     // compliance with the License.  You may obtain a copy of the License
     // at http://www.mozilla.org/MPL/
     //
     // Software distributed under the License is distributed on an "AS IS"
     // basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
     // See the License for the specific language governing rights and
     // limitations under the License.
     //
     // The Original Code is "Mini".
     //
     // The Initial Developer of the Original Code is David Wallace Croft
     // (croft@alumni.caltech.edu, http://www.alumni.caltech.edu/~croft/).
     // Portions created by David Wallace Croft are
     // Copyright (C) 1999 David Wallace Croft.  All Rights Reserved.
     //
     // Contributor(s):
     //////////////////////////////////////////////////////////////////////

     package com.orbs.open.a.mpl.compiler.mini.node;

     import java.util.*;

     /*********************************************************************
     * Parse tree node for the Mini programming language.
     *
     * @see
     *   MiniNode
     *
     * @author
     *   <A HREF="http://www.alumni.caltech.edu/~croft/">David W. Croft</A>
     * @version
     *   1999-04-26
     *********************************************************************/

     public class  ParameterSequenceMiniNode extends AbstractMiniNode
     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     {

     protected NameSequenceMiniNode  nameSequenceMiniNode;

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     /*********************************************************************
     * @throws  SemanticErrorException
     *   If there are duplicate parameter names.
     *********************************************************************/
     public  ParameterSequenceMiniNode (
       NameSequenceMiniNode  nameSequenceMiniNode )
       throws SemanticErrorException
     //////////////////////////////////////////////////////////////////////
     {
       this.nameSequenceMiniNode = nameSequenceMiniNode;

       List  nameMiniNodeList
         = nameSequenceMiniNode.getNameMiniNodeList ( );

       boolean  hasDuplicates = false;

       Set  nameSet = new HashSet ( );

       Iterator  i = nameMiniNodeList.iterator ( );
       while ( i.hasNext ( ) )
       {
         NameMiniNode  nameMiniNode = ( NameMiniNode ) i.next ( );

         String  name = nameMiniNode.getName ( );

         if ( nameSet.contains ( name ) )
         {
           hasDuplicates = true;
           break;
         }
         else
         {
           nameSet.add ( name );
         }
       }

       if ( hasDuplicates )
       {
         throw new SemanticErrorException (
           "duplicated parameter names" );
       }
     }

     //////////////////////////////////////////////////////////////////////
     // Access methods
     //////////////////////////////////////////////////////////////////////

     public NameSequenceMiniNode  getNameSequenceMiniNode ( )
     //////////////////////////////////////////////////////////////////////
     {
       return nameSequenceMiniNode;
     }

     public boolean  declares ( NameMiniNode  nameMiniNode )
     //////////////////////////////////////////////////////////////////////
     {
       return nameSequenceMiniNode.contains ( nameMiniNode );
     }

     public int  size ( )
     //////////////////////////////////////////////////////////////////////
     {
       return nameSequenceMiniNode.size ( );
     }

     //////////////////////////////////////////////////////////////////////
     // MiniNode interface methods
     //////////////////////////////////////////////////////////////////////

     public void  generate ( MiniNodeCodeVisitor  miniNodeCodeVisitor )
     //////////////////////////////////////////////////////////////////////
     {
       miniNodeCodeVisitor.generateParameterSequence ( this );
     }
     
     public void  checkSemantics ( Stack  parentMiniNodeStack )
       throws SemanticErrorException
     //////////////////////////////////////////////////////////////////////
     {
       parentMiniNodeStack.push ( this );

       nameSequenceMiniNode.checkSemantics ( parentMiniNodeStack );

       parentMiniNodeStack.pop ( );
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     }
